#!/usr/bin/env python
# vim: sts=4 sw=4 et

from icalendar import Calendar, Todo
from conics.tree import Tree

from conics.cache import CalCache, remote_uri

def isvtype(name):
    return lambda x: x.name == name

class TodoTree:
    def __init__(self, cal, depth = -1, cache = CalCache()):
        tree = Tree()
        pending = Tree()
        for t in filter(isvtype("VTODO"), cal.walk()):
            uid = t.decoded('uid')
            rel = t.decoded('related-to', None)
            remote = remote_uri(t)
            if remote and depth != 0:
                rc = cache.open(*remote)
                rt = TodoTree(rc, depth = depth - 1, cache = cache).tree
            else:
                rt = None
            if uid in pending:
                pending[uid] = t
                if rt:
                    pending[uid].merge(rt)
                if rel in tree: # None always in tree :)
                    tree.graft(rel, pending.pop(uid))
                else:
                    pending[rel] = None
                    pending.graft(rel, pending.pop(uid))
            elif rel in tree:
                tree.under(rel, uid, t)
                if rt:
                    tree[uid].merge(rt)
            else:
                if not rel in pending:
                    pending[rel] = None
                pending.under(rel, uid, t)
                if rt:
                    pending[uid].merge(rt)
	for t in pending.d.values():
            t.key = None
	    tree.merge(t)
        self.tree = tree

    def lookup(self, idl):
        tt = self.tree
        for idx in idl:
            try:
                idx = int(idx)
            except:
                raise ValueError("Components of ID chain must be integers: %s" % idx)
            k = sorted(tt.d.keys())
            if len(k) < idx:
                raise ValueError("No such todo: %s" % ".".join(idl))
            tt = tt.d[k[idx - 1]]
        return tt

